/*-------------------------------------+-------------------------------------*/
/*                                  board.c                                  */

#define NULL 0
enum boolean {FALSE, TRUE};

enum player  {WHITE, BLACK};
enum object  {PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING};


struct field;

struct piece {
   player dark;
   object type;
   field * loc;
};

struct field {
   piece * occ;
   field * pnext[2][2];
   field * xnext[2][2];
   field * Nnext[8];
   field * Bnext[4];
   field * Rnext[4];
   field * QKnext[8];
};

struct state {
   boolean OOwhite  : 1;
   boolean OOOwhite : 1;
   boolean OOblack  : 1;
   boolean OOOblack : 1;
   field * epfield;
};


piece piecelist[2][16];
field board[64];
state specials;
player color;


void init (void)
{
   unsigned i;

   for (i=0; i<16; i++) {
      piecelist[WHITE][i].dark = WHITE;
      piecelist[BLACK][i].dark = BLACK;
   }
   piecelist[WHITE][4].type = piecelist[BLACK][4].type = KING;
   piecelist[WHITE][3].type = piecelist[BLACK][3].type = QUEEN;
   piecelist[WHITE][0].type = piecelist[WHITE][7].type = ROOK;
   piecelist[BLACK][0].type = piecelist[BLACK][7].type = ROOK;
   piecelist[WHITE][2].type = piecelist[WHITE][5].type = BISHOP;
   piecelist[BLACK][2].type = piecelist[BLACK][5].type = BISHOP;
   piecelist[WHITE][1].type = piecelist[WHITE][6].type = KNIGHT;
   piecelist[BLACK][1].type = piecelist[BLACK][6].type = KNIGHT;

   for (i=0; i<64; i++) {
      board[i].pnext[WHITE][0] = (i/8<7) ? board+i+8 : NULL;
      board[i].pnext[WHITE][1] = (i/8==1) ? board+i+16 : NULL;
      board[i].pnext[BLACK][0] = (0<i/8) ? board+i-8 : NULL;
      board[i].pnext[BLACK][1] = (i/8==6) ? board+i-16 : NULL;
      board[i].xnext[WHITE][0] = (i/8<7 && 0<i%8) ? board+i+7 : NULL;
      board[i].xnext[WHITE][1] = (i/8<7 && i%8<7) ? board+i+9 : NULL;
      board[i].xnext[BLACK][0] = (0<i/8 && 0<i%8) ? board+i-9 : NULL;
      board[i].xnext[BLACK][1] = (0<i/8 && i%8<7) ? board+i-7 : NULL;
      board[i].Nnext[0] = (i/8<6 && 0<i%8) ? board+i+15 : NULL;
      board[i].Nnext[1] = (i/8<6 && i%8<7) ? board+i+17 : NULL;
      board[i].Nnext[2] = (i/8<7 && 1<i%8) ? board+i+ 6 : NULL;
      board[i].Nnext[3] = (i/8<7 && i%8<6) ? board+i+10 : NULL;
      board[i].Nnext[4] = (0<i/8 && 1<i%8) ? board+i-10 : NULL;
      board[i].Nnext[5] = (0<i/8 && i%8<6) ? board+i- 6 : NULL;
      board[i].Nnext[6] = (1<i/8 && 0<i%8) ? board+i-17 : NULL;
      board[i].Nnext[7] = (1<i/8 && i%8<7) ? board+i-15 : NULL;
      board[i].Bnext[0] = (i/8<7 && 0<i%8) ? board+i+7 : NULL;
      board[i].Bnext[1] = (i/8<7 && i%8<7) ? board+i+9 : NULL;
      board[i].Bnext[2] = (0<i/8 && 0<i%8) ? board+i-9 : NULL;
      board[i].Bnext[3] = (0<i/8 && i%8<7) ? board+i-7 : NULL;
      board[i].Rnext[0] = (0<i%8) ? board+i-1 : NULL;
      board[i].Rnext[1] = (i/8<7) ? board+i+8 : NULL;
      board[i].Rnext[2] = (i%8<7) ? board+i+1 : NULL;
      board[i].Rnext[3] = (0<i/8) ? board+i-8 : NULL;
      board[i].QKnext[0] = (i/8<7 && 0<i%8) ? board+i+7 : NULL;
      board[i].QKnext[1] = (i/8<7) ? board+i+8 : NULL;
      board[i].QKnext[2] = (i/8<7 && i%8<7) ? board+i+9 : NULL;
      board[i].QKnext[3] = (0<i%8) ? board+i-1 : NULL;
      board[i].QKnext[4] = (i%8<7) ? board+i+1 : NULL;
      board[i].QKnext[5] = (0<i/8 && 0<i%8) ? board+i-9 : NULL;
      board[i].QKnext[6] = (0<i/8) ? board+i-8 : NULL;
      board[i].QKnext[7] = (0<i/8 && i%8<7) ? board+i-7 : NULL;
   }
   return;
}


boolean attacked (field * f)
{
   unsigned i;

   for (i=0; i<2; i++)
      if (f->xnext[color][i] && f->xnext[color][i]->occ &&
         f->xnext[color][i]->occ->dark != color &&
         f->xnext[color][i]->occ->type == PAWN)
         return TRUE;
   for (i=0; i<8; i++)
      if (f->Nnext[i] && f->Nnext[i]->occ &&
         f->Nnext[i]->occ->dark != color &&
         f->Nnext[i]->occ->type == KNIGHT)
         return TRUE;
   for (i=0; i<4; i++) {
      field * f0;
      for (f0=f->Bnext[i]; f0 && ! f0->occ; f0=f0->Bnext[i])
         ;
      if (f0 && f0->occ->dark != color &&
         (f0->occ->type==BISHOP || f0->occ->type==QUEEN))
         return TRUE;
   }
   for (i=0; i<4; i++) {
      field * f0;
      for (f0=f->Rnext[i]; f0 && ! f0->occ; f0=f0->Rnext[i])
         ;
      if (f0 && f0->occ->dark != color &&
         (f0->occ->type==ROOK || f0->occ->type==QUEEN))
         return TRUE;
   }
   for (i=0; i<8; i++)
      if (f->QKnext[i] && f->QKnext[i]->occ &&
         f->QKnext[i]->occ->dark != color &&
         f->QKnext[i]->occ->type == KING)
         return TRUE;
   return FALSE;
}

