/*-------------------------------------+-------------------------------------*/
/*                                 movelist.c                                */

#include "hardbord.h"

boolean hardboard::escape (piece * p0, field * f1, object promotion)
{
   push (p0, f1, promotion);
   color = (color) ? WHITE : BLACK;
   boolean result = (attacked (piecelist[color][4].loc)) ? FALSE : TRUE;
   color = (color) ? WHITE : BLACK;
   pop ();
   return result;
}

boolean hardboard::ismate (void)
{
   unsigned i, j;

   for (i=0; i<16; i++)
      if (piecelist[color][i].loc) {
         piece * p0 = piecelist[color]+i;
         field * f1;
         switch (p0->type) {
         case PAWN:
            f1 = p0->loc->pnext[color][0];
            if (f1 && ! f1->occ) {
               if (board+8<=f1 && f1<board+56) {
                  if (escape (p0, f1)) return FALSE;
               } else {
                  if (escape (p0, f1, QUEEN )) return FALSE;
               }
               f1 = p0->loc->pnext[color][1];
               if (f1 && ! f1->occ)
                  if (escape (p0, f1)) return FALSE;
            }
            for (j=0; j<2; j++) {
               f1 = p0->loc->xnext[color][j];
               if (f1 &&
                  (f1->occ && f1->occ->dark!=color || f1==specials.epfield))
                  if (board+8<=f1 && f1<board+56) {
                     if (escape (p0, f1)) return FALSE;
                  } else {
                     if (escape (p0, f1, QUEEN )) return FALSE;
                  }
            }
            break;
         case KNIGHT:
            for (j=0; j<8; j++) {
               f1 = p0->loc->Nnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color))
                  if (escape (p0, f1)) return FALSE;
            }
            break;
         case BISHOP:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Bnext[j]; f1 && ! f1->occ; f1=f1->Bnext[j])
                  if (escape (p0, f1)) return FALSE;
               if (f1 && f1->occ->dark!=color)
                  if (escape (p0, f1)) return FALSE;
            }
            break;
         case ROOK:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Rnext[j]; f1 && ! f1->occ; f1=f1->Rnext[j])
                  if (escape (p0, f1)) return FALSE;
               if (f1 && f1->occ->dark!=color)
                  if (escape (p0, f1)) return FALSE;
            }
            break;
         case QUEEN:
            for (j=0; j<8; j++) {
               for (f1=p0->loc->QKnext[j]; f1 && ! f1->occ; f1=f1->QKnext[j])
                  if (escape (p0, f1)) return FALSE;
               if (f1 && f1->occ->dark!=color)
                  if (escape (p0, f1)) return FALSE;
            }
            break;
         case KING:
            for (j=0; j<8; j++) {
               f1 = p0->loc->QKnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color))
                  if (escape (p0, f1)) return FALSE;
            }
            break;
         }
      }
   return TRUE;
}

void hardboard::trial (piece * p0, field * f1, object promotion)
{
   boolean check = FALSE;
   boolean mate  = FALSE;

   push (p0, f1, promotion);
   color = (color) ? WHITE : BLACK;
   boolean incheck = attacked (piecelist[color][4].loc);
   color = (color) ? WHITE : BLACK;
   if (! incheck)
      if (attacked (piecelist[color][4].loc)) {
         check = TRUE;
         mate  = ismate ();
      }
   pop ();
   if (! incheck) {
      move ** m;
      for (m=&movelist; *m; m=&(*m)->next)
         ;
      *m = new move;
      (*m)->p0 = p0;
      (*m)->f1 = f1;
      (*m)->promotion = promotion;
      (*m)->check = check;
      (*m)->mate = mate;
      (*m)->next = NULL;
   }
   return;
}

void hardboard::make_movelist (void)
{
   move * helpm;
   for (helpm=movelist; helpm; helpm=movelist) {
      movelist = movelist->next;
      delete helpm;
   }

   unsigned i, j;
   for (i=0; i<16; i++)
      if (piecelist[color][i].loc) {
         piece * p0 = piecelist[color]+i;
         field * f1;
         switch (p0->type) {
         case PAWN:
            f1 = p0->loc->pnext[color][0];
            if (f1 && ! f1->occ) {
               if (board+8<=f1 && f1<board+56)
                  trial (p0, f1);
               else {
                  trial (p0, f1, QUEEN);
                  trial (p0, f1, ROOK);
                  trial (p0, f1, BISHOP);
                  trial (p0, f1, KNIGHT);
               }
               f1 = p0->loc->pnext[color][1];
               if (f1 && ! f1->occ)
                  trial (p0, f1);
            }
            for (j=0; j<2; j++) {
               f1 = p0->loc->xnext[color][j];
               if (f1 &&
                  (f1->occ && f1->occ->dark!=color || f1==specials.epfield))
                  if (board+8<=f1 && f1<board+56)
                     trial (p0, f1);
                  else {
                     trial (p0, f1, QUEEN);
                     trial (p0, f1, ROOK);
                     trial (p0, f1, BISHOP);
                     trial (p0, f1, KNIGHT);
                  }
            }
            break;
         case KNIGHT:
            for (j=0; j<8; j++) {
               f1 = p0->loc->Nnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color))
                  trial (p0, f1);
            }
            break;
         case BISHOP:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Bnext[j]; f1 && ! f1->occ; f1=f1->Bnext[j])
                  trial (p0, f1);
               if (f1 && f1->occ->dark!=color)
                  trial (p0, f1);
            }
            break;
         case ROOK:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Rnext[j]; f1 && ! f1->occ; f1=f1->Rnext[j])
                  trial (p0, f1);
               if (f1 && f1->occ->dark!=color)
                  trial (p0, f1);
            }
            break;
         case QUEEN:
            for (j=0; j<8; j++) {
               for (f1=p0->loc->QKnext[j]; f1 && ! f1->occ; f1=f1->QKnext[j])
                  trial (p0, f1);
               if (f1 && f1->occ->dark!=color)
                  trial (p0, f1);
            }
            break;
         case KING:
            if (color) {
               if (specials.OOOblack &&
                  ! board[59].occ && ! board[58].occ && ! board[57].occ &&
                  ! attacked (board+60) && ! attacked (board+59))
                  trial (p0, board+58);
               if (specials.OOblack  &&
                  ! board[61].occ && ! board[62].occ &&
                  ! attacked (board+60) && ! attacked (board+61))
                  trial (p0, board+62);
            } else {
               if (specials.OOOwhite &&
                  ! board[3].occ && ! board[2].occ && ! board[1].occ &&
                  ! attacked (board+4) && ! attacked (board+3))
                  trial (p0, board+2);
               if (specials.OOwhite  &&
                  ! board[5].occ && ! board[6].occ &&
                  ! attacked (board+4) && ! attacked (board+5))
                  trial (p0, board+6);
            }
            for (j=0; j<8; j++) {
               f1 = p0->loc->QKnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color))
                  trial (p0, f1);
            }
            break;
         }
      }

   n_moves = 0;
   for (helpm=movelist; helpm; helpm=helpm->next)
      n_moves++;
   return;
}

