/*-------------------------------------+-------------------------------------*/
/*                                 softbord.c                                */

#include "board.h"

struct mind {
   long value;
   piece * p0;
   field * f1;
   object promotion;
};

class softboard {
   long (*evaluate) (void);
   unsigned max, depth;
   mind * root, * best;
   inline long worst (void);
   void trial (piece *, field *, object = PAWN);
   long minimax (void);
public:
   softboard (long (*)(void));
   boolean treesearch (unsigned);
   mind result;
   ~softboard (void);
};

const long big = 1000000L;

softboard::softboard (long (*eval)(void))
{
   evaluate = eval;
   return;
}

inline long softboard::worst (void)
{
   return (color) ? +big-max+depth : -big+max-depth;
}

static inline boolean better (long a, long b)
{
   return (color) ? (a<b ? TRUE : FALSE) : (a>b ? TRUE : FALSE);
}

static inline void movepiece (field * f0, field * f1)
{
   f1->occ = f0->occ;
   f1->occ->loc = f1;
   f0->occ = NULL;
}

void softboard::trial (piece * p0, field * f1, object promotion)
{
   if (f1==specials.epfield && p0->type==PAWN)
      movepiece (f1->pnext[!color][0], f1);

   field * f0 = p0->loc;
   piece * p1 = f1->occ;

   if (p0->type == KING) {
      if (f0==board+ 4) {
         if (f1 == board+ 2) movepiece (board+ 0, board+ 3);
         if (f1 == board+ 6) movepiece (board+ 7, board+ 5);
      }
      if (f0==board+60) {
         if (f1 == board+58) movepiece (board+56, board+59);
         if (f1 == board+62) movepiece (board+63, board+61);
      }
   }

   state old_specials = specials;
   if (specials.OOwhite  && (f0==board+ 4 || f0==board+ 7 || f1==board+ 7))
      specials.OOwhite  = FALSE;
   if (specials.OOOwhite && (f0==board+ 4 || f0==board+ 0 || f1==board+ 0))
      specials.OOOwhite = FALSE;
   if (specials.OOblack  && (f0==board+60 || f0==board+63 || f1==board+63))
      specials.OOblack  = FALSE;
   if (specials.OOOblack && (f0==board+60 || f0==board+56 || f1==board+56))
      specials.OOOblack = FALSE;
   if (p0->type)
      specials.epfield = NULL;
   else
      if (color==WHITE && f0<board+16 && f1>=board+24 ||
         color==BLACK && f0>=board+48 && f1<board+40)
         specials.epfield = f0->pnext[color][0];
      else
         specials.epfield = NULL;

   f0->occ = NULL;
   f1->occ = p0;
   p0->loc = f1;
   if (p1)
      p1->loc = NULL;
   if (promotion)
      p0->type = promotion;

   if (! attacked (piecelist[color][4].loc)) {
      long value = worst ();

      best++, depth--;
      color = (color) ? WHITE : BLACK;

      if (depth==1 && better (evaluate(), (best-1)->value) &&
         ! attacked (piecelist[color][4].loc))
         ;  /* Razor Forward Pruning */
      else
         value = (depth) ? minimax () : (*evaluate) ();

      color = (color) ? WHITE : BLACK;
      best--, depth++;

      if (! best->p0 || better (value, best->value)) {
         best->value = value;
         best->p0 = p0;
         best->f1 = f1;
         best->promotion = promotion;
      }
   }

   f0->occ = p0;
   f1->occ = p1;
   p0->loc = f0;
   if (p1)
      p1->loc = f1;
   if (promotion)
      p0->type = PAWN;

   specials = old_specials;

   if (p0->type == KING) {
      if (f0==board+ 4) {
         if (f1 == board+ 2) movepiece (board+ 3, board+ 0);
         if (f1 == board+ 6) movepiece (board+ 5, board+ 7);
      }
      if (f0==board+60) {
         if (f1 == board+58) movepiece (board+59, board+56);
         if (f1 == board+62) movepiece (board+61, board+63);
      }
   }

   if (f1==specials.epfield && p0->type==PAWN)
      movepiece (f1, f1->pnext[!color][0]);

   return;
}

#define AB_TEST { for (mind * ab=best; ab>root; ab-=2) \
                     if (better (best->value, (ab-1)->value)) \
                        return best->value; }

long softboard::minimax (void)
{
   best->p0 = NULL;
   best->value = worst ();

   unsigned i, j;
   for (i=0; i<16; i++)
      if (piecelist[color][i].loc) {
         piece * p0 = piecelist[color]+i;
         field * f1;
         switch (p0->type) {
         case PAWN:
            f1 = p0->loc->pnext[color][0];
            if (f1 && ! f1->occ) {
               if (board+8<=f1 && f1<board+56) {
                  trial (p0, f1);  AB_TEST;
               } else {
                  trial (p0, f1, QUEEN);  AB_TEST;
                  trial (p0, f1, ROOK);   AB_TEST;
                  trial (p0, f1, BISHOP); AB_TEST;
                  trial (p0, f1, KNIGHT); AB_TEST;
               }
               f1 = p0->loc->pnext[color][1];
               if (f1 && ! f1->occ) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            for (j=0; j<2; j++) {
               f1 = p0->loc->xnext[color][j];
               if (f1 &&
                  (f1->occ && f1->occ->dark!=color || f1==specials.epfield))
                  if (board+8<=f1 && f1<board+56) {
                     trial (p0, f1);  AB_TEST;
                  } else {
                     trial (p0, f1, QUEEN);  AB_TEST;
                     trial (p0, f1, ROOK);   AB_TEST;
                     trial (p0, f1, BISHOP); AB_TEST;
                     trial (p0, f1, KNIGHT); AB_TEST;
                  }
            }
            break;
         case KNIGHT:
            for (j=0; j<8; j++) {
               f1 = p0->loc->Nnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color)) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            break;
         case BISHOP:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Bnext[j]; f1 && ! f1->occ; f1=f1->Bnext[j]) {
                  trial (p0, f1);  AB_TEST;
               }
               if (f1 && f1->occ->dark!=color) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            break;
         case ROOK:
            for (j=0; j<4; j++) {
               for (f1=p0->loc->Rnext[j]; f1 && ! f1->occ; f1=f1->Rnext[j]) {
                  trial (p0, f1);  AB_TEST;
               }
               if (f1 && f1->occ->dark!=color) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            break;
         case QUEEN:
            for (j=0; j<8; j++) {
               for (f1=p0->loc->QKnext[j]; f1 && ! f1->occ; f1=f1->QKnext[j]) {
                  trial (p0, f1);  AB_TEST;
               }
               if (f1 && f1->occ->dark!=color) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            break;
         case KING:
            if (color) {
               if (specials.OOOblack &&
                  ! board[59].occ && ! board[58].occ && ! board[57].occ &&
                  ! attacked (board+60) && ! attacked (board+59)) {
                  trial (p0, board+58);  AB_TEST;
               }
               if (specials.OOblack  &&
                  ! board[61].occ && ! board[62].occ &&
                  ! attacked (board+60) && ! attacked (board+61)) {
                  trial (p0, board+62);  AB_TEST;
               }
            } else {
               if (specials.OOOwhite &&
                  ! board[3].occ && ! board[2].occ && ! board[1].occ &&
                  ! attacked (board+4) && ! attacked (board+3)) {
                  trial (p0, board+2);  AB_TEST;
               }
               if (specials.OOwhite  &&
                  ! board[5].occ && ! board[6].occ &&
                  ! attacked (board+4) && ! attacked (board+5)) {
                  trial (p0, board+6);  AB_TEST;
               }
            }
            for (j=0; j<8; j++) {
               f1 = p0->loc->QKnext[j];
               if (f1 && (! f1->occ || f1->occ->dark!=color)) {
                  trial (p0, f1);  AB_TEST;
               }
            }
            break;
         }
      }

   if (! best->p0 && ! attacked (piecelist[color][4].loc))
      best->value = 0;
   return best->value;
}

boolean softboard::treesearch (unsigned maxdepth)
{
   boolean move;
   if (maxdepth) {
      root = best = new mind [max=depth=maxdepth];
      minimax ();
      result = best[0];
      move = (result.p0) ? TRUE : FALSE;
      delete root;
   } else {
      result.value = (*evaluate) ();
      move = FALSE;
   }
   return move;
}

softboard::~softboard (void)
{
   return;
}

